package sugar;

import javax.swing.*;
import java.awt.image.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;

/**
 * <p>Display</p>
 * <p>This class enables simple drawing <p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: FEL CVUT</p>
 * @author Jiri Danecek & Martin Bloch
 * @version 2.0
 */

public class Display extends JFrame {

  /**
   * Inner class
   */
  class Canvas extends JPanel {

    Dimension dim;                      // size of canvas
    BufferedImage bi;

    /**
     *
     */
    public Dimension getPreferredSize() { return dim; }

    /**
     * Cinstructor
     */
    Canvas(int width, int height) {
      this.setDoubleBuffered(false);
      bi = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
      dim = new Dimension(width, height);
    }

    public void paint(Graphics g) { g.drawImage(bi, 0, 0, null); }
  }

  // ******************** ATTRIBUTES  ***************
  private Canvas canvas;
  private Graphics2D g2;
  int width, height;
  private double lastX, lastY;
  private static Display self;               //


  /**
   *
   * @param size
   * @return
   */
  public static Display getDisplay(int size) { return getDisplay(size, ""); }

  /**
   *
   * @param size
   * @param title
   * @return
   */
  public static Display getDisplay(int size, String title) {
    return getDisplay(size, size, title);
  }

  /**
   *
   * @param width
   * @param height
   * @return
   */
  public static Display getDisplay(int width, int height) {
    return getDisplay(width, height, "");
  }

  /**
   *
   * @param width
   * @param height
   * @param title
   * @return
   */
  public static Display getDisplay(int width, int height, String title) {
    if (self != null) return self;
    return new Display(width, height, title);
  }

  /**
   *
   */
  public static void free() {
    if (self != null) { self.dispose(); self = null; }
  }

  /**
   * Constructor
   * @param width
   * @param height
   * @param title
   */
  private Display(int width, int height, String title) {
    self = this;
    setTitle(title);
    setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
    setResizable(false);
    canvas = new Canvas(width, height);
    g2 = (Graphics2D)canvas.bi.getGraphics();
    getContentPane().add(canvas, BorderLayout.CENTER);

    GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
    GraphicsDevice gd =  ge.getDefaultScreenDevice();
    GraphicsConfiguration gc =gd.getDefaultConfiguration();
    Rectangle displayBounds = gc.getBounds();

    this.width  = Math.min(width,  displayBounds.width);
    this.height = Math.min(height, displayBounds.height);
    setBounds((displayBounds.width-width)/2, (displayBounds.height-height)/2, width, height);
    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent we) { dispose(); self = null; }
    });
    pack();
    setVisible(true);
  }

  /**
   * Move cursor to position
   * @param x
   * @param y
   */
  public void moveCursor(double x, double y) { lastX = x;  lastY = y; }

  /**
   * Draw text on specified position using given size.
   * @param x
   * @param y
   * @param text
   * @param fontSize
   */
  public void drawString(double x, double y, String text, int fontSize) {
    g2.setFont(g2.getFont().deriveFont(Font.PLAIN, fontSize));
    g2.drawString(text, (int)x, (int)y);
    canvas.repaint();
  }

  /**
   * Draw text on specified position using given size and color.
   * @param x
   * @param y
   * @param text
   * @param fontSize
   * @param color
   */
  public void drawString(double x, double y, String text, int fontSize, Color color) {
    setColor(color);
    drawString(x, y, text, fontSize);
  }

  /**
   *
   * @param x
   * @param y
   */
  public void drawPoint(double x, double y) {
    Rectangle2D.Double r2d= new Rectangle2D.Double(x,y,1,1);
    g2.draw(r2d);
    canvas.repaint((int)x,(int)y,1,1);
  }

  /**
   * Draw point on specified position using given color.
   * @param x
   * @param y
   * @param c
   */
  public void drawPoint(double x, double y, Color c) {
    setColor(c);
    drawPoint(x, y);
  }

  /**
   * Draw line from present position to specified position.
   * @param x
   * @param y
   */
  public void drawLine(double x, double y) { drawLine(x, y, Color.white); }

  /**
   * Draw line from present position to specified position using given color.
   * @param x
   * @param y
   * @param c
   */
  public void drawLine(double x, double y, Color c) {
    setColor(c);
    drawLine(lastX, lastY, x, y);
    moveCursor(x, y);
  }

  /**
   * Draw line from position 1 to position 2.
   * @param x1
   * @param y1
   * @param x2
   * @param y2
   */
  public void drawLine(double x1, double y1, double x2, double y2) {
    moveCursor(x2, y2);
    Line2D.Double l2d = new Line2D.Double(x1, y1, x2, y2);
    Rectangle r=l2d.getBounds();
    g2.draw(l2d);
    canvas.repaint(r.x, r.y, r.width+1, r.height+1);
  }

  /**
   * Draw line from position 1 to position 2 using given color.
   * @param x1
   * @param y1
   * @param x2
   * @param y2
   * @param color
   */
  public void drawLine(double x1, double y1, double x2, double y2, Color color) {
    setColor(color);
    drawLine(x1, y1, x2, y2);
  }


  /**
   *
   * @param x
   * @param y
   * @param w
   * @param h
   */
  public void drawRect(double x, double y, double w, double h) {
    moveCursor(x, y);
    Rectangle2D.Double r2d = new Rectangle2D.Double(x, y, w, h);
    Rectangle r=r2d.getBounds();
    g2.draw(r2d);
    canvas.repaint(r.x, r.y, r.width+1, r.height+1);
  }

  /**
   *
   * @param x
   * @param y
   * @param w
   * @param h
   * @param color
   */
  public void drawRect(double x, double y, double w, double h, Color color) {
    setColor(color);
    drawRect(x, y, w, h);
  }

  /**
   *
   * @param x
   * @param y
   * @param w
   * @param h
   */
  public void fillRect(double x, double y, double w, double h) {
    moveCursor(x, y);
    Rectangle2D.Double r2d = new Rectangle2D.Double(x, y, w, h);
    Rectangle r=r2d.getBounds();
    g2.fill(r2d);
    canvas.repaint(r.x, r.y, r.width+1, r.height+1);
  }

  /**
   *
   * @param x
   * @param y
   * @param w
   * @param h
   * @param color
   */
  public void fillRect(double x, double y, double w, double h, Color color) {
    setColor(color);
    fillRect(x, y, w, h);
  }

  /**
   *
   * @param x
   * @param y
   * @param radius
   */
  public void drawCircle(double x, double y, double radius) {
    drawCircle(x, y, radius, Color.white);
  }

  /**
   *
   * @param x
   * @param y
   * @param radius
   * @param color
   */
  public void drawCircle(double x, double y, double radius, Color color) {
    g2.setColor(color);
    moveCursor(x, y);
    int xx=(int)(x-radius), yy=(int)(y-radius), wh=(int)(2*radius);
    g2.drawOval(xx, yy, wh, wh);
    canvas.repaint(xx,yy,wh+1,wh+1);
  }

  /**
   *
   * @param x
   * @param y
   * @param radius
   */
  public void fillCircle(double x, double y, double radius) {
    fillCircle(x, y, radius, Color.white);
  }

  /**
   *
   * @param x
   * @param y
   * @param radius
   * @param color
   */
  public void fillCircle(double x, double y, double radius, Color color) {
    g2.setColor(color);
    moveCursor(x, y);
    int xx=(int)(x-radius), yy=(int)(y-radius), wh=(int)(2*radius);
    g2.fillOval(xx, yy, wh, wh);
    canvas.repaint(xx,yy,wh,wh);
  }

  /**
   *
   * @param col
   */
  public void setColor(Color col) { g2.setColor(col); }

  /**
   *
   * @param title
   */
  public void setTitle(String title) { super.setTitle(title); }

  /**
   *
   * @param c
   */
  public void clear(Color c) {
    Graphics2D g = (Graphics2D)g2.create();
    g.setColor(c);
    g.fillRect(0, 0, width, height);
    canvas.repaint();
  }

  /**
   *
   */
  public void clear() {
    Graphics2D g = (Graphics2D)g2.create();
    clear(g2.getBackground());
  }

  /**
   *
   * @return
   */
  public int width() { return canvas.dim.width; }

  /**
   *
   * @return
   */
  public int height() { return canvas.dim.height; }

  // ***************************************************************************
  // TESTER

  public static void main(String[] args) {
    Display d = Display.getDisplay(500);
    d.clear(Color.magenta);
    d.drawLine(0, 0, 10, 10);

    d.drawLine(10, 10, 0, 10);
    d.drawLine(0, 10, 4, 2);
    d.drawLine(4, 2, 10, 10);
    d.fillCircle(100,100,50,Color.cyan);
    d.fillRect(200,200,100,50,Color.white);
    d.fillRect(200,200,50,40,Color.red);
    d.fillRect(200,200,30,30,Color.white);
  }
}